from PyQt5 import uic
from PyQt5.QtWidgets import QDialog
import os.path
from ..paggf_layers import LoadedLayers
from ..paggf_utils import PAGGFUtils
from ..composer.composer_utils import load_from_template
import tempfile
from qgis.core import QgsComposerHtml, QgsComposerFrame
from PyQt5.QtGui import QPageSize, QDesktopServices
from PyQt5.QtCore import QUrl

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'CID_munip_list.ui'))


class CIDMunipListDlg(QDialog, FORM_CLASS):
    def __init__(self, parent=None):
        super(CIDMunipListDlg, self).__init__(parent)
        self.setupUi(self)
        self.layers = LoadedLayers()
        self.utils = PAGGFUtils(self.layers)
        self.parcel_layer = self.layers.get('gis', 'current_idc_parcels')
        self.fields_aliases_dict = self.parcel_layer.attributeAliases()
        self.fields_aliases_dict_inv = {
            v: k for k, v in self.fields_aliases_dict.items()}

        self.default_selection_name = [
            'begin_date', 'idc_parcel_code', 'hill_district_name',
            'sub_hill_avenue_name', 'usage', 'conflicting_parts'
            # todo manque : superficie, riverin
        ]

        self.do_not_propose_name_for_selection = [
            'b_id', 'id_polygon', 'id_split_origin_parcel', 'id_inventory',
            'is_polygon_valid', 'polygon_invalidity', 'id'
        ]

        if self.parcel_layer:
            for f in self.parcel_layer.dataProvider().fields():
                f_name = f.name()
                if f_name not in self.default_selection_name and\
                        f_name not in self.do_not_propose_name_for_selection:
                    self.fields_for_selection.addItem(
                        self.alias_for_feature(f))

        for n in self.default_selection_name:
            self.selected_fields.addItem(self.alias_or_name(n))

        self.buttonBox.accepted.connect(self.accepted_slot)

    def alias_for_feature(self, f):
        """
        Get the alias for a field. If the alias does not exists the function
        return the name of the field.
        """
        return self.alias_or_name(f.name())

    def alias_or_name(self, name):
        """
        Given a name return the alias or the name itseft if if does not exists
        """
        if name in self.fields_aliases_dict:
            return self.fields_aliases_dict[name]
        return name

    def name_from_alias(self, alias):
        """
        Get the field name from the alias.
        """
        if alias in self.fields_aliases_dict_inv:
            return self.fields_aliases_dict_inv[alias]
        else:  # pas eu de conversion par fct alias_or_name, donc alias = name
            return alias

    def accepted_slot(self):
        seleted_fields_alias = []

        for i in range(len(self.selected_fields)):
            seleted_fields_alias.append(self.selected_fields.item(i).text())

        selected_field_names = [
            self.name_from_alias(a) for a in seleted_fields_alias]

        array_thead_data = seleted_fields_alias

        municipality = self.utils.get_current_municipality()
        province = self.utils.get_current_province()
        html_content = "<table><thead><tr>"

        html_content += "<th>N°</th>"

        for alias in seleted_fields_alias:
            html_content += "<th>{}</th>".format(alias)

        html_content += "</tr></thead>"
        html_content += "<tbody>"

        it = 1

        for p in self.utils.get_current_parcels_for(municipality):
            html_content += "<tr>"

            html_content += "<td>{}</td>".format(it)

            for f_name in selected_field_names:
                html_content += "<td>{}</td>".format(p[f_name])

            html_content += "</tr>"

            it += 1

        html_content += "</tbody>"
        html_content += "</table>"

        composition = load_from_template(
            'CID_A3_municipality_list.qpt',
            substitution_map={
                'QGIS_PLUGIN_DIR': self.utils.plugin_dir,
                'province': province['name'],
                'municipality': municipality['name']}
        )

        html_item = QgsComposerHtml(composition, False)

        paper_size = QPageSize.size(QPageSize.A3, QPageSize.Millimeter)
        margin_bottom = 10
        margin_top = 70
        margin_left = 10
        paper_width = paper_size.height()  # landscape mode
        paper_height = paper_size.width()  # landscape mode

        html_frame = QgsComposerFrame(
            composition,
            html_item,
            margin_left,
            margin_top,
            paper_width - 2 * margin_left,
            paper_height - margin_top - margin_bottom)

        html_frame.setItemPosition(
            margin_left,
            margin_top,
            page=1
        )

        html_item.addFrame(html_frame)

        html_item.setResizeMode(QgsComposerHtml.ExtendToNextPage)

        html_item.setContentMode(QgsComposerHtml.ManualHtml)

        html_item.setUserStylesheet("\
table {\
    border-collapse: collapse;\
    empty-cells: show;\
}\
\
table, th, td {\
    border: 1px solid black;\
}")

        html_item.setUserStylesheetEnabled(True)
        html_item.setHtml(html_content)
        html_item.loadHtml()

        out_path = os.path.join(
            tempfile.gettempdir(), "list_munip.pdf")

        composition.exportAsPDF(out_path)

        QDesktopServices.openUrl(QUrl.fromLocalFile(out_path))
